import 'package:flutter/material.dart';

class VPopupRoute<T> extends PopupRoute<T> {
  final Widget child;
  final Offset? offsetLT, offsetRB;
  final Duration duration;
  final bool cancelable;
  final bool outsideTouchCancelable;
  final double opacity;
  final List<RRect>? highlights;
  final VoidCallback? onPopupShow;
  final VoidCallback? onPopupDismiss;

  VPopupRoute(
      {required this.child,
      this.offsetLT,
      this.offsetRB,
      this.cancelable = true,
      this.outsideTouchCancelable = true,
      this.opacity = 0.6,
      this.duration = const Duration(milliseconds: 300),
      this.highlights,
      this.onPopupShow,
      this.onPopupDismiss}) {
    assert(opacity <= 1.0 && opacity >= 0.0);
  }

  @override
  // Color get barrierColor => Colors.black.withAlpha(127);
  // Color get barrierColor => Color(0x80000000);
  Color get barrierColor => Colors.transparent;

  @override
  bool get barrierDismissible => false;

  @override
  String get barrierLabel => "";

  @override
  TickerFuture didPush() {
    onPopupShow?.call();
    return super.didPush();
  }

  @override
  bool didPop(T? result) {
    onPopupDismiss?.call();
    return super.didPop(result);
  }

  @override
  Widget buildPage(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation) {
    return _PopRouteWidget(
      child: child,
      offsetLT: offsetLT,
      offsetRB: offsetRB,
      duration: duration,
      cancelable: cancelable,
      outsideTouchCancelable: outsideTouchCancelable,
      opacity: opacity,
      highlights: highlights,
    );
  }

  @override
  Duration get transitionDuration => duration;

  static pop(BuildContext context) {
    __PopRouteWidgetState.of(context)?.dismiss();
    Navigator.of(context).pop();
  }

  static setHighlights(BuildContext context, List<RRect> highlights) {
    __PopRouteWidgetState.of(context)?.highlights = highlights;
  }
}

class _PopRouteWidget extends StatefulWidget {
  final Widget? child; //Popup弹出的内容Widget
  final Offset? offsetLT, offsetRB; //背景区域范围的left、top、right、bottom
  final Duration? duration;
  final bool cancelable;
  final bool outsideTouchCancelable;
  final double opacity;
  final List<RRect>? highlights;

  _PopRouteWidget({
    this.child,
    this.offsetLT,
    this.offsetRB,
    this.duration,
    this.cancelable = true,
    this.outsideTouchCancelable = true,
    this.opacity = 1,
    this.highlights,
  });

  @override
  __PopRouteWidgetState createState() => __PopRouteWidgetState();
}

class __PopRouteWidgetState extends State<_PopRouteWidget>
    with SingleTickerProviderStateMixin {
  late Animation<double> alphaAnim;
  late AnimationController alphaController;
  List<RRect> _highlights = [];

  @override
  void initState() {
    super.initState();
    highlights = widget.highlights ?? [];
    alphaController = AnimationController(
      vsync: this,
      duration: widget.duration,
    );
    alphaAnim = Tween<double>(begin: 0, end: 255).animate(alphaController);
    alphaController.forward();
  }

  static __PopRouteWidgetState? of(BuildContext context) {
    return context.findAncestorStateOfType<__PopRouteWidgetState>();
  }

  dismiss() {
    alphaController.reverse();

    // widget.child?.onPopupDismiss();
  }

  set highlights(List<RRect> value) {
    setState(() {
      _highlights = value;
    });
  }

  @override
  void dispose() {
    alphaController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        if (widget.cancelable) {
          dismiss();
          return true;
        }
        return false;
      },
      child: GestureDetector(
        onTap: () {
          if (widget.outsideTouchCancelable) {
            Navigator.of(context).pop();
            dismiss();
          }
        },
        child: Stack(
          children: <Widget>[
            widget.opacity != 0
                ? AnimatedBuilder(
                    animation: alphaController,
                    builder: (_, __) {
                      //经过Padding控制变暗的范围区域
                      return Padding(
                        padding: EdgeInsets.only(
                          left: widget.offsetLT?.dx ?? 0,
                          top: widget.offsetLT?.dy ?? 0,
                          right: widget.offsetRB?.dx ?? 0,
                          bottom: widget.offsetRB?.dy ?? 0,
                        ),
                        child: ColorFiltered(
                          colorFilter: ColorFilter.mode(
                            Colors.black.withAlpha(
                                (alphaAnim.value * widget.opacity).toInt()),
                            BlendMode
                                .srcOut, //暗色蒙层为src，srcOut即展示蒙层与子组件不相交的部分，效果即为在蒙层上把子组件部分全部镂空
                          ),
                          child: Stack(
                            children: _buildDark(),
                          ),
                        ),
                      );
                    },
                  )
                : Container(),
            widget.child ?? Container(),
          ],
        ),
      ),
    );
  }

  List<Widget> _buildDark() {
    List<Widget> widgets = [];
    widgets.add(Container(
      color: Colors.transparent,
    ));

    //根据RRect区域生成需要镂空的子组件
    for (RRect highlight in _highlights) {
      widgets.add(Positioned(
        child: Container(
          decoration: BoxDecoration(
              color: Colors.redAccent,
              border: Border.all(width: 5, color: Colors.blue),
              borderRadius: BorderRadius.only(
                topLeft: highlight.tlRadius,
                topRight: highlight.trRadius,
                bottomLeft: highlight.blRadius,
                bottomRight: highlight.brRadius,
              )),
          width: highlight.width,
          height: highlight.height,
        ),
        left: highlight.left,
        top: highlight.top,
      ));
    }
    return widgets;
  }
}
